<?php

namespace HS\IncomingMail\Jobs;

use HS\Mailbox;
use Illuminate\Support\Str;
use Illuminate\Bus\Queueable;
use Illuminate\Support\Facades\Log;

class MailClerk
{
    use Queueable;

    protected $debug;

    private $mailbox_ids;

    /**
     * Create a new job instance.
     *
     * @param array $mailbox_ids
     */
    public function __construct($mailbox_ids = [])
    {
        $this->mailbox_ids = $mailbox_ids;
    }

    /**
     * Grab the mail and dispatch to FetchMail.
     */
    public function handle()
    {
        $this->loadTheWorld();

        $this->debug = hs_setting('cHD_TASKSDEBUG', false);

        $this->setLicense();

        $mailboxes = Mailbox::getActive();

        foreach ($mailboxes as $box) {

            //Only do specific mailbox if IDs are passed in
            if (count($this->mailbox_ids) > 0 && ! in_array($box->xMailbox, $this->mailbox_ids)) {
                continue;
            }

            if (! $box->mailboxIsBusy()) {
                // Generate a transaction id to help trace log entries
                $transaction = Str::uuid()->toString();
                Log::debug('['.get_class($this).'] Dispatching FetchMail', [
                    'xMailbox' => $box->xMailbox,
                    'transaction' => $transaction,
                ]);

                FetchMail::dispatch($box, $transaction); // mail.incoming
            } else {
                Log::debug('['.get_class($this).'] Mailbox is busy, not dispatching FetchMail', [
                    'xMailbox' => $box->xMailbox,
                ]);
            }
        }

        return true;
    }

    /**
     * Set the global license.
     */
    protected function setLicense()
    {
        $licenseObj = new \usLicense(hs_setting('cHD_CUSTOMER_ID'), hs_setting('cHD_LICENSE'), hs_setting('SSKEY'));
        $GLOBALS['license'] = $licenseObj->getLicense();
    }

    /**
     * Load the world.
     */
    protected function loadTheWorld()
    {
        ob_start();

        /*****************************************
        INCLUDE PATH
         *****************************************/
        set_include_path(cBASEPATH.'/helpspot/pear');

        /*****************************************
        INCLUDE LIBS
         *****************************************/
        require_once cBASEPATH.'/helpspot/lib/utf8.lib.php';
        require_once cBASEPATH.'/helpspot/lib/util.lib.php';
        require_once cBASEPATH.'/helpspot/lib/error.lib.php';
        require_once cBASEPATH.'/helpspot/lib/platforms.lib.php';
        require_once cBASEPATH.'/helpspot/pear/Mail/helpspot_mimeDecode.php';
        require_once cBASEPATH.'/helpspot/lib/api.lib.php';
        require_once cBASEPATH.'/helpspot/lib/display.lib.php';
        require_once cBASEPATH.'/helpspot/lib/api.users.lib.php';
        require_once cBASEPATH.'/helpspot/lib/api.requests.lib.php';
        require_once cBASEPATH.'/helpspot/lib/class.notify.php';
        require_once cBASEPATH.'/helpspot/lib/api.mailboxes.lib.php';
        require_once cBASEPATH.'/helpspot/lib/class.requestupdate.php';
        require_once cBASEPATH.'/helpspot/lib/class.userscape.bayesian.classifier.php';
        require_once cBASEPATH.'/helpspot/lib/class.mail.rule.php';
        require_once cBASEPATH.'/helpspot/pear/Console/Getopt.php';
        require_once cBASEPATH.'/helpspot/lib/class.business_hours.php';
        require_once cBASEPATH.'/helpspot/lib/class.language.php';
        require_once cBASEPATH.'/helpspot/lib/phpass/PasswordHash.php';
        require_once cBASEPATH.'/helpspot/lib/class.license.php';
        require_once cBASEPATH.'/helpspot/lib/lookup.lib.php';	//include lookups here so we can use lang abstraction
        ob_clean();
    }
}
